import { Component, InputHTMLAttributes, PureComponent, FC, HTMLAttributes, ChangeEvent, MouseEvent, DragEvent, KeyboardEvent, FormEvent, ReactNode } from "react";
import { ImageProps, FloatingLabelProps, FormControlProps, TableProps } from "react-bootstrap";
import { BaseModel, DataObject, IDType, ListModel, Filter } from "mobx-restful";
import { TranslationModel } from "mobx-i18n";
export interface FormComponentProps extends Omit<InputHTMLAttributes<HTMLInputElement>, 'onChange'> {
    onChange?: (value: InputHTMLAttributes<HTMLInputElement>['value'], ...extra: any[]) => any;
}
export abstract class FormComponent<P extends FormComponentProps = FormComponentProps, S = {}, SS = any> extends Component<P, S, SS> {
    ref: import("react").RefObject<HTMLInputElement & HTMLTextAreaElement & HTMLSelectElement>;
    innerValue: P["defaultValue"];
    get value(): P["defaultValue"] | P["value"];
    reset: () => void;
    componentDidMount(): void;
    componentWillUnmount(): void;
}
export class ImagePreview extends PureComponent<ImageProps> {
    downloading: boolean;
    loadedPath: string;
    viewing: boolean;
    componentDidMount(): void;
    componentDidUpdate({ src }: Readonly<ImageProps>): void;
    load(path: string): Promise<void>;
    render(): JSX.Element;
}
export type FilePreviewProps = ImageProps & HTMLAttributes<HTMLAudioElement> & HTMLAttributes<HTMLVideoElement> & HTMLAttributes<HTMLAnchorElement> & {
    type?: InputHTMLAttributes<HTMLInputElement>['accept'];
    path: string;
};
export const FilePreview: FC<FilePreviewProps>;
export interface FilePickerProps extends FormComponentProps {
    onChange?: (value: FormComponentProps['value'], file?: File) => any;
}
export class FilePicker extends FormComponent<FilePickerProps> {
    static readonly displayName = "FilePicker";
    handleAdd: (event: ChangeEvent<HTMLInputElement>) => void;
    handleClear: (event: MouseEvent<HTMLButtonElement>) => void;
    renderInput(): JSX.Element;
    render(): JSX.Element;
}
export abstract class FileModel extends BaseModel {
    files: string[];
    clear(): void;
    /**
     * Override this method for Network calling,
     * then call `super.upload(fileURL)` to update `this.files` array.
     */
    upload(file: string | Blob): Promise<string>;
    /**
     * Override this method for Network calling,
     * then call `super.delete(fileURL)` to update `this.files` array.
     */
    delete(file: string): Promise<void>;
    move(sourceIndex: number, targetIndex: number): void;
}
export interface FileUploaderProps extends FormComponentProps {
    store: FileModel;
}
export class FileUploader extends FormComponent<FileUploaderProps> {
    pickIndex?: number;
    componentDidMount(): void;
    handleDrop: (index: number) => (event: DragEvent<HTMLElement>) => void;
    handleChange: (oldURI?: string) => (URI: string, file?: File) => Promise<void>;
    render(): JSX.Element;
}
export type FormFieldProps = InputHTMLAttributes<HTMLInputElement> & FormControlProps & Pick<FloatingLabelProps, 'label'>;
export const FormField: FC<FormFieldProps>;
export const TextInputTypes: readonly ["text", "number", "tel", "email", "url"];
export interface BadgeInputProps extends Pick<InputHTMLAttributes<HTMLInputElement>, 'className' | 'style' | 'name' | 'required' | 'placeholder'> {
    type?: (typeof TextInputTypes)[number];
    defaultValue?: string[];
    value?: string[];
    onChange?: (value: string[]) => any;
}
export class BadgeInput extends PureComponent<BadgeInputProps> {
    static match(type: string): type is BadgeInputProps['type'];
    innerValue: string[];
    get value(): string[];
    handleInput: (event: KeyboardEvent<HTMLInputElement>) => void;
    delete(index: number): void;
    render(): JSX.Element;
}
export interface Field<T extends DataObject> extends Pick<InputHTMLAttributes<HTMLInputElement>, 'type' | 'accept'> {
    key?: keyof T;
    renderLabel?: ReactNode | ((data: keyof T) => ReactNode);
    renderInput?: (data: T) => ReactNode;
}
export interface RestFormProps<T extends DataObject> {
    id?: IDType;
    fields: Field<T>[];
    store: ListModel<T>;
    translator: TranslationModel<string, 'submit' | 'cancel'>;
}
export class RestForm<T extends DataObject> extends PureComponent<RestFormProps<T>> {
    componentDidMount(): void;
    handleSubmit: (event: FormEvent<HTMLFormElement>) => Promise<void>;
    renderInput: ({ key, renderLabel, renderInput, ...props }: Field<T>) => {};
    render(): JSX.Element;
}
export type PageMeta = Pick<ListModel<{}>, 'pageSize' | 'pageIndex'>;
export interface PagerProps extends PageMeta {
    pageCount: number;
    onChange?: (meta: PageMeta) => any;
}
export const Pager: FC<PagerProps>;
export interface Column<T extends DataObject> extends Omit<Field<T>, 'renderLabel'> {
    renderHead?: Field<T>['renderLabel'];
    renderBody?: (data: T) => ReactNode;
    renderFoot?: ReactNode | ((data: keyof T) => ReactNode);
}
export interface RestTableProps<T extends DataObject> extends TableProps, Omit<RestFormProps<T>, 'id' | 'fields' | 'translator'> {
    editable?: boolean;
    deletable?: boolean;
    columns: Column<T>[];
    translator: RestFormProps<T>['translator'] & TranslationModel<string, 'create' | 'edit' | 'delete' | 'total_x_rows' | 'sure_to_delete_x'>;
    onCheck?: (keys: IDType[]) => any;
}
export class RestTable<T extends DataObject> extends PureComponent<RestTableProps<T>> {
    static displayName: string;
    checkedKeys: IDType[];
    toggleCheck(key: IDType): void;
    toggleCheckAll: () => void;
    get checkColumn(): Column<T>;
    get operateColumn(): Column<T>;
    get columns(): Column<T>[];
    get hasHeader(): boolean;
    get hasFooter(): boolean;
    get editing(): boolean;
    componentDidMount(): void;
    renderTable(): JSX.Element;
    renderDialog(): JSX.Element;
    getList: import("lodash").DebouncedFunc<({ pageIndex, pageSize }: any) => void>;
    deleteList(keys: IDType[]): Promise<void>;
    render(): JSX.Element;
}
export type EdgePosition = 'top' | 'bottom' | 'left' | 'right';
export type TouchHandler = (edge: EdgePosition) => any;
export interface ScrollBoundaryProps extends Partial<Record<EdgePosition, ReactNode>> {
    className?: string;
    onTouch: TouchHandler;
}
export const ScrollBoundary: FC<ScrollBoundaryProps>;
export interface ScrollListProps<T extends DataObject = DataObject> extends Pick<ScrollBoundaryProps, 'className'> {
    defaultData?: T[];
}
export type DataType<P> = P extends ScrollListProps<infer D> ? D : never;
export abstract class ScrollList<P extends ScrollListProps> extends Component<P> {
    abstract store: ListModel<DataType<P>>;
    abstract translator: TranslationModel<string, 'load_more' | 'no_more'>;
    filter: Filter<DataType<P>>;
    boot(): Promise<void>;
    componentWillUnmount(): void;
    loadMore: import("lodash").DebouncedFunc<(edge: EdgePosition) => void>;
    abstract renderList(): ReactNode;
    render(): JSX.Element;
}

//# sourceMappingURL=index.d.ts.map
